/*
    SPDX-FileCopyrightText: 2014 Cristian Oneț <onet.cristian@gmail.com>
    SPDX-License-Identifier: GPL-2.0-or-later
*/

#ifndef MYMONEY_UNIT_TESTABLE
#define MYMONEY_UNIT_TESTABLE

#ifndef KMM_MYMONEY_UNIT_TESTABLE
#  define KMM_MYMONEY_UNIT_TESTABLE
#endif

#endif
